function [sigma,imat,nor] = fdemdoi(FDEMresults,threshold)
%FDEMDOI compute the depth of investigation (DOI) for a data vector/matrix 
%   and insert NaNs to highlite it.
%   [sigma,imat,nor] = fdemdoi(FDEMresults,threshold)
%   where
%      FDEMresults - data structure containing the results
%      threshold   - threshold percentage
%   and
%       sigma   - array of the electrical conductivity which contains NaN
%               under the depth of investigation
%       imat    - array of index layer for which the variation of the
%               sensitivity Is negligible with respect to the sensitivity 
%               at the surface
%       nor     - norm of the columns of the Jacobian matrix
%
%   Last revised January 26, 2023

sigma = FDEMresults.solution;
func = FDEMresults.model;

[n,ncol] = size(sigma);

nor = zeros(n,ncol);
imat = zeros(n,ncol);
for j = 1:ncol
	[~,J] = func(sigma(:,j));
	J(:,end) = 0;
	for i = 1:n
		nor(i,j) = norm(J(:,i));
	end
	ivet = nor(:,j) <= threshold*nor(1,j);
	sigma(ivet,j) = nan;
	imat(:,j) = ivet;
end

