function h = fdemplot(z,sigma,whattocomp,zf,fig)
%FDEMPLOT plot the recovered solution and, if avail., the exact one
%   h = fdemplot(z,sigma,zf,fig)
%   where
%      z          - horizontal axis
%      sigma      - reconstructed conductivity in each layer (S/m)
%      whattocomp - 1 for sigma, 2 for mu
%      zf         - end of the last layer
%      fig        - figure in which make the plot

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised September 6, 2019

n = size(z,1);
if nargin<5,  fig = gcf;  end
if nargin<4,  zf = 2*z(n)-z(n-1);  end
if nargin<3,  whattocomp = 0;  end

zz = zeros(n,1);
zz(2*n+[0;1]) = [zf;zf];
zz(1:2:2*n-1) = z;
zz(2:2:2*n-2) = z(2:n);
k = size(sigma,2);
ss = zeros(n,k);
for j = 1:k
	ss(2*n+1,j) = sigma(n,j);
	ss(1:2:2*n-1,j) = sigma(:,j);
	ss(2:2:2*n,j) = sigma(:,j);
end

figure(fig)
set(gcf,'NumberTitle','off','name','Solution');
hh = plot( zz, ss);
xlim([0 zf])
switch whattocomp
	case 0
		xlabel('')
		ylabel('')
	case 1
		xlabel('Depth (m)','fontsize',11)
		ylabel('Electrical conductivity (S/m)','fontsize',11)
	case 2
		xlabel('Depth (m)','fontsize',11)
		ylabel('Magnetic permeability (H/m)','fontsize',11)
end

if nargout>0,  h = hh;  end


