%FORWARDCOMP compute the forward model

%   G.P. Deidda, P. Díaz de Alba, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised January 26, 2023

if ~exist('app','dir'), app.stop = 0; end

fdemprint( ' ', calledbyapp, textarea)
switch modeltype
	case 1
		fdemprint( 'Quadrature component of the signal (scaled)', calledbyapp, textarea)
		type = 'apparent conductivity';
	case 2
		fdemprint( 'Quadrature component of the signal', calledbyapp, textarea)
		type = 'imaginary';
	case 3
		fdemprint( 'Inphase component of the signal', calledbyapp, textarea)
		type = 'real';
	case 4
		fdemprint( 'Complex signal (inphase + quadrature)', calledbyapp, textarea)
		type = 'complex (inphase + quadrature)';
	case 5
		fdemprint( 'Complex signal (inphase + apparent conductivity)', calledbyapp, textarea)
		type = 'scaled complex (inphase + apparent conductivity)';
end

if strcmpi(HVorient(1:2),'vp')|| strcmpi(HVorient(1:2),'pe')
	Rp = R + 0.1;
else
	Rp = R;
end

if geninputdata
	sigma = funsigma(z);
	mu = funmu(z);
	mur = mu/mu0;
	FDEMforward.name = 'Generated input models';
else
	clear n z lastz
	if ~fileavail
		if exist('dataforward','dir') == 7
			dirnam = 'dataforward/*.mat';
		else
			dirnam = '*.mat';
		end
		[filnam,pathnam,filtind] = uigetfile(dirnam,'Open data file');
		fullpathname = [pathnam filnam];
	end
	load(fullpathname)
	str = sprintf('Field data: %s', FDEMmodel.name);
	fdemprint(str, calledbyapp, textarea);

	sigma = FDEMmodel.sigma;
	mu = FDEMmodel.mu;
	mur = mu/mu0;
	z = FDEMmodel.z;
end

n = length(sigma);
lastz = z(end);
d = (lastz/(n-1))*ones(n-1,1);
nR = length(R);			% Number of coil pairs
nf = length(freq);		% Number of frequencies
omega = 2*pi*freq;		% Vector of angular frequencies

delta = skindepth(d,freq,sigma,mu);
% possibile modifica
if (m > 1) && (nR > 1)
	B = max(R,h');
	B = max(B')./delta;
	if strcmpi(HVorient(1:2),'vp')
		Bp = max(Rp,h');
		Bp = max(Bp')./delta;
	end
else
	if (m == 1) && (nR > 1) && (nf > 1)
		B = max(R,h')'./delta;
	else
		B = max(R,h')./delta;
	end
	if strcmpi(HVorient(1:2),'vp')
		Bp = max(Rp,h')./delta;
	end
end

%
% Instrument response for freq
%
switch modeltype		% define the model to invert
	case 1
		func = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 2
		func = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 3
		func = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 4
		func = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcmu = @(sigma,mu) hratio(sigma,mu,h,d,R,omega,HVorient,'mu');
	case 5
		funcinphase = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcacond = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'sigma');
		funcinphasemu = @(sigma,mu) inphase(sigma,mu,h,d,R,omega,HVorient,'mu');
		funcacondmu = @(sigma,mu) aconduct(sigma,mu,h,d,R,omega,HVorient,'mu');
end
funcd = @(sigma,mu,d) hratio(sigma,mu,h,d,R,omega,HVorient,'sigma');
funcdmu = @(sigma,mu,d) hratio(sigma,mu,h,d,R,omega,HVorient,'mu');

str = sprintf('orientation: %s\nfreq=', HVorient);
str = [str sprintf('%d ', freq)];
str = [str sprintf('\nR=')];
str = [str sprintf('%g ', R)];
str = [str sprintf('\nnh=%d, nR=%d, nf=%d, ', m, nR, nf)];
fdemprint( str, calledbyapp, textarea)

if modeltype == 5
	re = funcinphase(sigma,mu);
	im = funcacond(sigma,mu);
	data = complex(re,im);
else
	data = func(sigma,mu);
end
mdata = length(data);
switch lower(HVorient(1:2))
	case 've'
		vdata = data;
		hdata = [];
		pdata = [];
	case 'ho'
		vdata = [];
		hdata = data;
		pdata = [];
	case 'pe'
		vdata = [];
		hdata = [];
		pdata = data;
	case 'vh'
		vdata = data(1:mdata/2);
		hdata = data(mdata/2+1:end);
		pdata = [];
	case 'vp'
		vdata = data(1:mdata/2);
		hdata = [];
		pdata = data(mdata/2+1:end);
	otherwise
		error('Unknown orientation.')
end

if profgraph
	figure(fff)
	set(gcf,'NumberTitle', 'off','name','Ground model')
	subplot(1,2,1)
	msig = max(sigma);
	if msig==0, msig = 1; end
	plot(z,sigma,'r')
	grid on
	axis tight
	%axis auto
	ylim([0 msig*1.2])
	ylabel('Conductivity (S/m)','fontsize',11);
	xlabel('Depth (m)','fontsize',11)
	view([90 90])
	subplot(1,2,2)
	mmu = max(mur);
	minmu = min(mur);
	plot(z,mur,'r')
	grid on
	axis tight
	%axis auto
	ylim([minmu*0.98 mmu*1.02])
	ylabel('Relative permeability','fontsize',11);
	xlabel('Depth (m)','fontsize',11)
	view([90 90])
end

freqmin = min(freq);	% Frequency vector for graphical analysis of response
freqmax = max(freq);
nff = 10000;
f = logspace(log10(freqmin),log10(freqmax),nff)';
deltaf = skindepth(d,f,sigma,mu);
if graphindnum
	if (nR > 1) && (nf == 1)
		figure(fff+1)
		set(gcf,'NumberTitle', 'off','name','Induction number')
		clf
		semilogx(R,B,'o:')
		if strcmpi(HVorient(1:2),'vp')
			hold on
			semilogx(Rp,Bp,'*--')
			legend('Induction number \rho','Induction number \rho_p')
		end
		grid on
		xlabel('Intercoil distance \rho (m)')
		ylabel('Induction number B')
	elseif (nf > 1) && (nR == 1)
		Bf = R./deltaf;
		figure(fff+1)
		set(gcf,'NumberTitle', 'off','name','Induction number')
		clf
		semilogx(f,Bf,freq,B,'o')
		if strcmpi(HVorient(1:2),'vp')
			Bfp = Rp./deltaf;
			hold on
			semilogx(f,Bfp,freq,Bp,'*')
			legend('','Induction number \rho','','Induction number \rho_p')
		end
		%xlim([freqmin freqmax])
		grid on
		xlabel('Frequency (Hz)')
		ylabel('Induction number B')
	elseif (nf > 1) && (nR > 1)
		figure(fff+1)
		set(gcf,'NumberTitle', 'off','name','Induction number')
		clf
		semilogx(R,B,'o:')
		if strcmpi(HVorient(1:2),'vp')
			hold on
			semilogx(Rp,Bp,'*--')
			legend('Induction number \rho','Induction number \rho_p')
		end
		grid on
		xlabel('Intercoil distance \rho (m)')
		ylabel('Induction number B')
	else
		%if ishandle(fff+1)
		%	close(figure(fff+1))
		%end
		str1 = sprintf('Induction number corresponding to R: %4.2f \n', B);
		fdemprint(str1, calledbyapp, textarea)
		if strcmpi(HVorient(1:2),'vp')
			str2 = sprintf('Induction number corresponding to Rp: %4.2f \n', Bp);
			fdemprint(str2, calledbyapp, textarea)
		end
		%fprintf('Induction number = %4.2f \n',B);
	end
end

if graphskindepth
	if nf > 1
		figure(fff+2)
		set(gcf,'NumberTitle', 'off','name','Skin depth')
		clf
		semilogx(f,deltaf,freq,delta,'o')
		%xlim([freqmin freqmax])
		grid on
		xlabel('Frequency (Hz)')
		ylabel('Skin depth (m)')
	else
		%if ishandle(fff+2)
		%	close(figure(fff+2))
		%end
		str = sprintf('Skin depth: %4.2f metri\n', delta);
		fdemprint( str, calledbyapp, textarea)
		%fprintf('Skin Depth = %4.2f metri\n',delta);
	end
end

if sensfunc
	sensitivityfunc(sigma,mu,z,lastz,h,d,R,omega,HVorient,modeltype,fff);
	drawnow
end

if cumresp
	if geninputdata
		lastzext = lastz;
		next = 31;
		threshold = 1e-2;
		flag = 1;
		while flag % extend the interval until the DOI is exceeded
			lastzext = 2*lastzext;
			zext = linspace(0,lastzext,next)';
			dext = (lastzext/(next-1))*ones(next-1,1);
			sigmaext = funsigma(zext);
			muext = funmu(zext);
			[~,Jer] = funcd(sigmaext,muext,dext);
			[~,Jei] = funcdmu(sigmaext,muext,dext);
			res = norm(Jer(:,end-1)+1i*Jei(:,end-1));
			nor = norm(Jer(:,1)+1i*Jei(:,1));
			flag = res > threshold*nor;
			%next, norm(Jer(:,end-1)+1i*Jei(:,end-1)), pause
		end
		for i = 1:next-1	% find DOI depth
			res = norm(Jer(:,i)+1i*Jei(:,i));
			if res < threshold*nor, break, end
		end
		lastzext = zext(i);
		next = 301;
		zext = linspace(0,lastzext,next)';
		dext = (lastzext/(next-1))*ones(next-1,1);
		sigmaext = funsigma(zext);
		muext = funmu(zext);
	else
		lastzext = lastz;
		next = n;
		zext = linspace(0,lastz,n)';
		dext = (lastz/(n-1))*ones(n-1,1);
		sigmaext = sigma;
		muext = mu;
	end
	cumulativeresp(sigmaext,muext,zext,lastzext,h,dext,R,omega,HVorient,modeltype,fff);
	drawnow
end

switch graphtype
	case 1
		nr = 200;
		rr = logspace(log10(rmin),log10(rmax),nr)';

		%
		% Instrument response for f
		%
		switch modeltype		% define the model to invert
			case 1, funcr = @(sigma,mu) aconduct(sigma,mu,h,d,rr,omega,HVorient,'sigma');
			case 2, funcr = @(sigma,mu) quadracomp(sigma,mu,h,d,rr,omega,HVorient,'sigma');
			case 3, funcr = @(sigma,mu) inphase(sigma,mu,h,d,rr,omega,HVorient,'sigma');
			case 4, funcr = @(sigma,mu) hratio(sigma,mu,h,d,rr,omega,HVorient,'sigma');
			case 5, funcinphaser = @(sigma,mu) inphase(sigma,mu,h,d,rr,omega,HVorient,'sigma');
				funcacondr = @(sigma,mu) aconduct(sigma,mu,h,d,rr,omega,HVorient,'sigma');
		end

		if modeltype == 5
			re = funcinphaser(sigma,mu);
			im = funcacondr(sigma,mu);
			datar = complex(re,im);
		else
			datar = funcr(sigma,mu);
		end

		if nf == 1
			datar = 1e3*datar;
			data = 1e3*data;
		end
		mdatar = length(datar);

		% plot figures
		den = nR; denx = nr;
		if strcmpi(HVorient(1:2),'vh')
			den = 2*den; denx = 2*denx;
		end
		if strcmpi(HVorient(1:2),'vp')
			den = 2*den; denx = 2*denx;
		end
		data = reshape(data,[mdata/den,den]).';
		datar = reshape(datar,[mdatar/denx,denx]).';
		datarows = size(data,1);
		datarrows = size(datar,1);
		if (nR == 1) && (nf > 1) && (m > 1)
			h1 = repmat(h,nf,1);
			freq1 = repmat(freq',m,1);
			freq1 = sort(freq1);
			legendCell = strcat('freq=',strtrim(cellstr(num2str(freq1(:)))),', h=',strtrim(cellstr(num2str(h1(:)))));
		else
			legendCell = strcat('freq=',strtrim(cellstr(num2str(freq(:)))),', h=',strtrim(cellstr(num2str(h(:)))));
		end
		xlab = 'Intercoil distance (m)';
		if modeltype == 1
			ylab1 = '\sigma_a (mS/m)';
			ylab2 = '';
		elseif modeltype == 4
			ylab1 = 'H_S/H_P (ppm)';
			ylab2 = 'H_S/H_P (ppm)';
		elseif modeltype == 5
			ylab1 = 'H_S/H_P (ppt)';
			ylab2 = '\sigma_a (mS/m)';
		else
			ylab1 = 'H_S/H_P (ppm)';
			ylab2 = '';
		end
		rrp = rr + 0.1;
		plotforward(fff+4,data,R,Rp,HVorient,xlab,ylab1,ylab2,legendCell,legendCell,linscale,rr,rrp,datar,legendavail,pos,modeltype);

	case 2
		nff = 200;
		ff = logspace(log10(fmin),log10(fmax),nff)';
		omegaf = 2*pi*ff;

		%
		% Instrument response for f
		%
		switch modeltype		% define the model to invert
			case 1, funcf = @(sigma,mu) aconduct(sigma,mu,h,d,R,omegaf,HVorient,'sigma');
			case 2, funcf = @(sigma,mu) quadracomp(sigma,mu,h,d,R,omegaf,HVorient,'sigma');
			case 3, funcf = @(sigma,mu) inphase(sigma,mu,h,d,R,omegaf,HVorient,'sigma');
			case 4, funcf = @(sigma,mu) hratio(sigma,mu,h,d,R,omegaf,HVorient,'sigma');
			case 5, funcinphasef = @(sigma,mu) inphase(sigma,mu,h,d,R,omegaf,HVorient,'sigma');
				funcacondf = @(sigma,mu) aconduct(sigma,mu,h,d,R,omegaf,HVorient,'sigma');
		end

		%data = 1e6*data;
		if modeltype == 5
			re = funcinphasef(sigma,mu);
			im = funcacondf(sigma,mu);
			dataf = complex(re,im);
		else
			dataf = funcf(sigma,mu);
		end
		%dataf = 1e6*dataf;

		if modeltype == 1 || modeltype == 5
			dataf = 1e3*dataf;
			data = 1e3*data;
		else
			%if nf == 1
			%    dataf = 1e3*dataf;
			%    data = 1e3*data;
			%else
			dataf = 1e6*dataf;
			data = 1e6*data;
			%end
		end
		mdataf = length(dataf);

		% plot figures
		if nR > 1
			if strcmpi(HVorient(1:2),'vh')
				data = [reshape(data(1:mdata/2),[mdata/(2*m*nR),nR*m]);reshape(data(mdata/2+1:end),[mdata/(2*m*nR),nR*m])];
				dataf = [reshape(dataf(1:mdataf/2),[mdataf/(2*m*nR),nR*m]);reshape(dataf(mdataf/2+1:end),[mdataf/(2*m*nR),nR*m])];
			elseif strcmpi(HVorient(1:2),'vp')
				data = [reshape(data(1:mdata/2),[mdata/(2*m*nR),nR*m]);reshape(data(mdata/2+1:end),[mdata/(2*m*nR),nR*m])];
				dataf = [reshape(dataf(1:mdataf/2),[mdataf/(2*m*nR),nR*m]);reshape(dataf(mdataf/2+1:end),[mdataf/(2*m*nR),nR*m])];
			else
				data = reshape(data,[mdata/(nR*m),nR*m]);
				dataf = reshape(dataf,[mdataf/(nR*m),nR*m]);
			end
		else
			den = m; denx = m;
			if strcmpi(HVorient(1:2),'vh')
				den = 2*den; denx = 2*denx;
			end
			if strcmpi(HVorient(1:2),'vp')
				den = 2*den; denx = 2*denx;
			end
			data = reshape(data,[mdata/den,den]);
			dataf = reshape(dataf,[mdataf/denx,denx]);
			if strcmpi(HVorient(1:2),'vh') || strcmpi(HVorient(1:2),'vp')
				data = [data(:,1:m);data(:,m+1:den)];
				dataf = [dataf(:,1:m);dataf(:,m+1:denx)];
			end
		end
		datarows = size(data,1);
		datafrows = size(dataf,1);
		if (nR > 1) && (nf == 1) && (m > 1)
			h1 = repmat(h,nR,1);
			R1 = repmat(R,m,1);
			R1 = sort(R1);
			if strcmpi(HVorient(1:2),'vp')
				R1p = repmat(Rp,m,1);
				R1p = sort(R1p);
				legendCellp = strcat('\rho=',strtrim(cellstr(num2str(R1p(:)))),', h=',strtrim(cellstr(num2str(h1(:)))));
			end
			legendCell = strcat('\rho=',strtrim(cellstr(num2str(R1(:)))),', h=',strtrim(cellstr(num2str(h1(:)))));
		elseif (nR > 1) && (nf > 1) && (m > 1)
			h1 = repmat(h,nR,1);
			R1 = repmat(R,m,1);
			R1 = sort(R1);
			if strcmpi(HVorient(1:2),'vp')
				R1p = repmat(Rp,m,1);
				R1p = sort(R1p);
				legendCellp = strcat('\rho=',strtrim(cellstr(num2str(R1p(:)))),', h=',strtrim(cellstr(num2str(h1(:)))));
			end
			legendCell = strcat('\rho=',strtrim(cellstr(num2str(R1(:)))),', h=',strtrim(cellstr(num2str(h1(:)))));
		else
			if strcmpi(HVorient(1:2),'vp')
				legendCellp = strcat('\rho=',strtrim(cellstr(num2str(Rp(:)))),', h=',strtrim(cellstr(num2str(h(:)))));
			end
			legendCell = strcat('\rho=',strtrim(cellstr(num2str(R(:)))),', h=',strtrim(cellstr(num2str(h(:)))));
		end
		xlab = 'Frequency (Hz)';
		if modeltype == 1
			ylab1 = '\sigma_a (mS/m)';
			ylab2 = '';
		elseif modeltype == 4
			ylab1 = 'H_S/H_P (ppm)';
			ylab2 = 'H_S/H_P (ppm)';
		elseif modeltype == 5
			ylab1 = 'H_S/H_P (ppt)';
			ylab2 = '\sigma_a (mS/m)';
		else
			ylab1 = 'H_S/H_P (ppm)';
			ylab2 = '';
		end
		if strcmpi(HVorient(1:2),'vp')
			plotforward(fff+4,data,freq,freq,HVorient,xlab,ylab1,ylab2,legendCell,legendCellp,linscale,ff,ff,dataf,legendavail,pos,modeltype);
		else
			plotforward(fff+4,data,freq,freq,HVorient,xlab,ylab1,ylab2,legendCell,legendCell,linscale,ff,ff,dataf,legendavail,pos,modeltype);
		end
	case 3
		nh = 200;
		hh = logspace(log10(hhmin),log10(hhmax),nh)';

		%
		% Instrument response for f
		%
		switch modeltype		% define the model to invert
			case 1, funch = @(sigma,mu) aconduct(sigma,mu,hh,d,R,omega,HVorient,'sigma');
			case 2, funch = @(sigma,mu) quadracomp(sigma,mu,hh,d,R,omega,HVorient,'sigma');
			case 3, funch = @(sigma,mu) inphase(sigma,mu,hh,d,R,omega,HVorient,'sigma');
			case 4, funch = @(sigma,mu) hratio(sigma,mu,hh,d,R,omega,HVorient,'sigma');
			case 5, funcinphaseh = @(sigma,mu) inphase(sigma,mu,hh,d,R,omega,HVorient,'sigma');
				funcacondh = @(sigma,mu) aconduct(sigma,mu,hh,d,R,omega,HVorient,'sigma');
		end

		%data = 1e3.*data;
		if modeltype == 5
			re = funcinphaseh(sigma,mu);
			im = funcacondh(sigma,mu);
			datah = complex(re,im);
		else
			datah = funch(sigma,mu);
		end
		%datah = 1e3.*datah;

		if nf == 1
			if modeltype == 1 || modeltype == 5
				datah = 1e3*datah;
				data = 1e3*data;
			else
				datah = 1e6*datah;
				data = 1e6*data;
			end
		end
		mdatah = length(datah);

		% plot figures
		if nR > 1
			den = nR; denx = nR;
			if strcmpi(HVorient(1:2),'vh'), den = 2*den; denx = 2*denx; end
			if strcmpi(HVorient(1:2),'vp'), den = 2*den; denx = 2*denx; end
			data = reshape(data,[mdata/den,den]);
			datah = reshape(datah,[mdatah/denx,denx]);
			if strcmpi(HVorient(1:2),'vh')
				data = [data(:,1:nR);data(:,nR+1:den)];
				datah = [datah(:,1:nR);datah(:,nR+1:denx)];
			end
			if strcmpi(HVorient(1:2),'vp')
				data = [data(:,1:nR);data(:,nR+1:den)];
				datah = [datah(:,1:nR);datah(:,nR+1:denx)];
			end
		else
			den = m; denx = nh;
			if strcmpi(HVorient(1:2),'vh')
				den = 2*den; denx = 2*denx;
			end
			if strcmpi(HVorient(1:2),'vp')
				den = 2*den; denx = 2*denx;
			end
			data = reshape(data,[mdata/den,den]).';
			datah = reshape(datah,[mdatah/denx,denx]).';
		end
		datarows = size(data,1);
		datahrows = size(datah,1);
		if strcmpi(HVorient(1:2),'vp')
			legendCellp = strcat('\rho=',strtrim(cellstr(num2str(Rp(:)))),', freq=',strtrim(cellstr(num2str(freq(:)))));
		end
		legendCell = strcat('\rho=',strtrim(cellstr(num2str(R(:)))),', freq=',strtrim(cellstr(num2str(freq(:)))));
		xlab = 'Height (m)';
		if modeltype == 1
			ylab1 = '\sigma_a (mS/m)';
			ylab2 = '';
		elseif modeltype == 4
			ylab1 = 'H_S/H_P (ppm)';
			ylab2 = 'H_S/H_P (ppm)';
		elseif modeltype == 5
			ylab1 = 'H_S/H_P (ppt)';
			ylab2 = '\sigma_a (mS/m)';
		else
			ylab1 = 'H_S/H_P (ppm)';
			ylab2 = '';
		end
		if strcmpi(HVorient(1:2),'vp')
			plotforward(fff+4,data,h,h,HVorient,xlab,ylab1,ylab2,legendCell,legendCellp,linscale,hh,hh,datah,legendavail,pos,modeltype);
		else
			plotforward(fff+4,data,h,h,HVorient,xlab,ylab1,ylab2,legendCell,legendCell,linscale,hh,hh,datah,legendavail,pos,modeltype);
		end
	otherwise
		error('Graph type unknown.')

end

