function k = morozov( rho, sigma, noisetype, normb, tau, phi)
%MOROZOV choose regularization parameter by the discrepancy principle
%   k = morozov( rho, sigma, noisetype, normb, tau, phi)
%   where
%      rho 	 - residual
%      sigma     - array of conductivities per layer (S/m)
%      noisetype - kind of noise
%      normb     - norm of the right-hand side b
%      tau  	 - user-suplied constant
%      phi  	 - 
%   and
%      k         - regularization parameter for Discrepancy Principle

%   G.P. Deidda, P. Díaz de Alba, C. Fenu, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised June 10, 2019

if nargin<6 || isempty(phi),  phi = 0;  end
if nargin<5 || isempty(tau),  tau = 1.3;  end

if strncmp(noisetype(1:3),'rel',3)
	%kv = find(rho < tau*(sigma*normb+phi));
	kv = find(rho.^2 < (tau*sigma*normb)^2+phi^2);
	if ~isempty(kv)
		k = kv(1);
	else
		k = [];
	end
else
	error( 'Noise type unknown.')
end

