function delta = skindepth(d,freq,sigma,mu)
%SKINDEPTH compute the skin depth
%   skindepth(d,freq,sigma,mu)
%   where
%	d	- thickness vector 
%       freq	- frequency vector
%       sigma	- electrical conductivity vector
%	mu	- magnetic permeability vector

%   G.P. Deidda, P. Díaz de Alba, and G. Rodriguez
%   University of Cagliari, Italy
%
%   Last revised January 25, 2021

omega = 2*pi*freq;
nomega = length(omega);

n = size(sigma,1);
sigmaN = sigma(end);
muN = mu(end);

delta = [];

for j = 1:nomega
    kN = sqrt(1i*omega(j)*muN*sigmaN);
    Cn = 1/kN;
    C = Cn;
    for i = n-1:-1:1
        k = sqrt(1i*omega(j)*mu(i)*sigma(i));
        r = (1-k*C)/(1+k*C);
        C = (1-r*exp(-2*k*d(i)))/(k*(1+r*exp(-2*k*d(i))));
    end
    delta = [delta ; sqrt(2)*abs(C)];
end
